/*
GenericPort.h

Virtual Base Class for Serial and TCP/IP Comm functions
*/

#pragma once

#include "Xpert/Win32Fix.h"
#include "ssp.h"

#ifdef NOT_XPERT
#include "Xpert/io.h"
#else
#include "XpertAPI.h"
#endif


#undef DECL
#if defined(NOT_XPERT)
#define DECL
#else
#if defined(_AFXDLL) || defined(XPERT_EXE)
#ifdef UTILS_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif
#else
#define DECL 
#endif
#endif

enum CPortType {ptUNKNOWN=0, ptRS232=1, ptTCPIP=2, ptREMOTE=3};

class DECL CGenericPort
{
public:
   CGenericPort()                   { TotalPut = TotalGet = 0; pRxData = NULL; LastCh = 0; CRCState = 0; };
   virtual ~CGenericPort()          { if (pRxData) delete pRxData;                         };

   // Sets the default value that will be used once the port is open
   virtual void SetTimeouts(COMMTIMEOUTS& CommTimeouts) {if (&m_CommTimeouts != &CommTimeouts) m_CommTimeouts = CommTimeouts;};

   // Sets the default value and (if possible) causes it to take effect right away
   virtual BOOL SetCommTimeouts(COMMTIMEOUTS& CommTimeouts) { SetTimeouts(CommTimeouts); return TRUE; };

   virtual int GetPortType() { return ptUNKNOWN; }; // 0=Unknown, 1=RS232, 2=TCPIP, 3=REMOTE SERIAL

   // open/close
   virtual bool CloseComm() = 0;
   virtual bool OpenComm()  = 0;
   virtual bool IsOpen()    = 0;

   virtual BOOL ResetWake() = 0; // Put the uart back to sleep after it's auto-woken up (restore's the power state)
   // data
#ifdef UNICODE
   bool GetChar(char* ch) {return GetChar((UCHAR *) ch);};
#endif
   virtual bool GetChar(UCHAR* uch) = 0;
   virtual bool GetChar(TCHAR* wch) = 0;

   virtual int GetStr(UCHAR* Str, int StrLen) = 0;

#ifdef UNICODE
   int GetStr(char*  Str, int StrLen) {return GetStr((UCHAR*)Str, StrLen);};
#endif

   virtual int GetStr(TCHAR* Str, int StrLen) = 0;
   virtual int Get(void* Buffer, int BufferLen) = 0;
   virtual int Get(void* Buffer, int BufferLen, DWORD dwTimeout) = 0;

   bool LineInput(CString& strResp, DWORD dwTimeout=5000, DWORD dwNumBytes=INFINITE);

   virtual bool Put(const void* Buffer, int BufferLen, DWORD dwTimeout) = 0;
   virtual bool PutAndWait(const void* Buffer, int BufferLen, DWORD dwTimeout) = 0;
   virtual bool Put(const void* Buffer, int BufferLen) = 0;
   virtual bool PutAndWait(const void* Buffer, int BufferLen) = 0;

#ifdef UNICODE
   bool PutChar(const char ch) {return PutChar((const UCHAR)ch);};
#endif

   virtual bool PutChar(const UCHAR uch) = 0;
   virtual bool PutChar(const TCHAR wch) = 0;

#ifdef UNICODE
   virtual bool PutStr(const char* Str)             { return PutStr((UCHAR*)Str);         };
   virtual bool PutStr(const char* Str, int StrLen) { return PutStr((UCHAR*)Str, StrLen); };
#endif

   virtual bool PutStr(const UCHAR* Str)            { return (PutStr(Str, strlen((char*)Str))); };
   virtual bool PutStr(TCHAR* Str)                  { return PutStr(Str, _tcsclen(Str));          };

   virtual bool PutStr(const UCHAR* Str, int StrLen) = 0;
   virtual bool PutStr(TCHAR* Str, int StrLen) = 0;
   virtual bool PutStr(CString Str) = 0;

   virtual int  NumberBytesInputBuffer() = 0; // returns number of bytes in input buffer
   virtual bool KeyPressed() = 0;             // return true if a key is in rx buffer
   virtual void FlushInput() = 0;             // empties rx buffer

   virtual void WaitOnRx() = 0;                   // waits (blocks) until a char comes in the serial port - flushes input first
   virtual void WaitForTxEmpty() = 0;             // waits (blocks) until transmit buffer empty.
   virtual void WaitForTxEmpty(TCHAR LastCh) = 0; // More accurate version of WaitForTxEmpty()

   virtual BOOL SetCommState(LPDCB lpDCB) = 0;
   virtual BOOL GetCommState(LPDCB lpDCB) = 0;
   virtual BOOL SetCommMask(DWORD dwEvtMask) = 0;
   virtual BOOL GetCommMask(LPDWORD dwEvtMask) = 0;
   virtual BOOL EscapeCommFunction(DWORD dwFunc) = 0;
   virtual BOOL SetCommBreak() = 0;
   virtual BOOL ClearCommBreak() = 0;
   virtual BOOL GetCommModemStatus(LPDWORD lpModemStat) = 0;
   virtual BOOL WaitCommEvent(LPDWORD lpEvtMask) = 0;

// SSP Read/Write
   virtual bool Read(TSSPPacket& SSPPacket, int Timeout);
   virtual bool Write(TSSPPacket& SSPPacket, int Timeout);
   virtual bool Read(TSSPPacket& SSPPacket);
   virtual bool Write(TSSPPacket& SSPPacket);

protected:
   COMMTIMEOUTS m_CommTimeouts;
   TData* pRxData;
   char LastCh;
   int CRCState;

public:
   DWORD TotalPut;
   DWORD TotalGet;
};
