// StorageCards.h : header file for StorageCards.cpp of Utils.dll
//

#if !defined(_STORAGE_CARD_INCLUDED_)
#define _STORAGE_CARD_INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#undef DECL
#if defined(_AFXDLL) || defined(XPERT_EXE)
#ifdef UTILS_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif
#else
#define DECL 
#endif


/////////////////////////////////////////////////////////////////////////////
// Storage card routines.
//
class DECL TStorageCards
{
   ////////////////////////////////
   // Interface
   ////////////////////////////////
   public:
   
   // Constructors and destructor.
   TStorageCards() {RefreshList();};
   virtual ~TStorageCards() {};

   // Enumeration of card types. 
   enum ECardTypes {eNO_CARDS = 0, eSD_CARD = 0x01, eUSB_CARD = 0x02, eCF_CARD = 0x04};
   
   // Determine which cards are present.
   void RefreshList();

   // Returns true if SD card is present.
   bool IsSDPresent() {return IsPresent(eSD_CARD);};
   
   // Returns true if USB card is present.
   bool IsUSBPresent() {return IsPresent(eUSB_CARD);};
   
   // Returns true if CF card is present.
   bool IsCFPresent() {return IsPresent(eCF_CARD);};
   
   // Returns true if the card(s) in dwCardList are present. Multiple cards can be 
   // specified by adding or or-ing enums together as in eSD_CARD + eUSB_CARD.
   bool IsPresent(ECardTypes ctCardList);
   bool IsPresent(DWORD dwCardList) {return IsPresent((ECardTypes)dwCardList);};

   // Return the enumerated list of installed card types.
   ECardTypes InstalledCards() {return m_ctCardList;};
   
   // Return number of installed cards. 
   int NumInstalled();

   // Assign one instance to another. 
   TStorageCards& operator = (const TStorageCards& sc) 
         {m_ctCardList = sc.m_ctCardList; return *this;};

   ////////////////////////////////
   // Implementation
   ////////////////////////////////
   protected:

   // List of current cards installed.
   ECardTypes m_ctCardList;
};


#endif // !defined(_STORAGE_CARD_INCLUDED_)
